// display.c
//
// 09-06-2003 23:19:32
//
// Copyright 2003 Terry Roberts
//
// See readme.html for more information.
//
// This file is part of MineSweeper.
// 
// MineSweeper is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// MineSweeper is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with MineSweeper; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

// C Source File
// Created 9/6/03; 12:45:52 AM

#include <tigcclib.h>         // Include All Header Files
#include "display.h"
#include "sprites.h"

void drawBoard()
{
	// vertical lines
	int x, y = 2, i, j;
	for (i = 0; i < 9; ++i)
	{
		x = 4;
		for (j = 0; j < 17; ++j)
		{
			Sprite8(x,y,8,virgin,LCD_MEM,A_REPLACE);
			x += 9;
		}
		y += 9;
	}
	short oldFont = FontSetSys(F_6x8);
	DrawStr(1,85,"P1 ",A_XOR);
	DrawStr(141,85," P2",A_XOR);
	FontSetSys(oldFont);
}

void drawPlayer(int p)
{
	switch (p)
	{
		case 1:
			Sprite8(62,85,8,lArrow,LCD_MEM,SPRT_XOR);
			Sprite8(90,85,8,clear,LCD_MEM,SPRT_AND);
			break;
		case 2:
			Sprite8(90,85,8,rArrow,LCD_MEM,SPRT_XOR);
			Sprite8(62,85,8,clear,LCD_MEM,SPRT_AND);
			break;
	}
}

void drawMinesLeft(int n)
{
	short oldFont = FontSetSys(F_6x8);
	char string[3];
	sprintf(string,"%d",n);
	if (n > 9) // is two digits
	{
		DrawChar(74,85,string[0],A_REPLACE);
		DrawChar(80,85,string[1],A_REPLACE);
	}
	else // is one digit
	{
		DrawChar(74,85,'0',A_REPLACE);
		DrawChar(80,85,string[0],A_REPLACE);
	}
	FontSetSys(oldFont);
}

void drawMinesFlagged(int p, int n)
{
	short oldFont = FontSetSys(F_6x8);
	char string[3];
	sprintf(string,"%d",n);
	switch (p)
	{
		case 1:
			if (n > 9)
			{
				DrawStr(20,85,string,A_REPLACE);
			}
			else
			{
				DrawChar(20,85,'0',A_REPLACE);
				DrawChar(26,85,string[0],A_REPLACE);
			}
			break;
		case 2:
			if (n > 9)
			{
				DrawStr(128,85,string,A_REPLACE);
			}
			else
			{
				DrawChar(128,85,'0',A_REPLACE);
				DrawChar(134,85,string[0],A_REPLACE);
			}
			break;
	}
	FontSetSys(oldFont);
}

void drawSprite(int xCoord, int yCoord, STATE s)
{
	int x = 4;
	x += 9 * xCoord;
	int y = 2;
	y += 9 * yCoord;
	unsigned char* sprite;
	switch (s)
	{
		case SELECTED:
			sprite = selected;
			break;
		case VIRGIN:
			sprite = virgin;
			break;
		case CLEAR:
			sprite = clear;
			break;
		case MINE:
			sprite = mine;
			break;
		case FLAG:
			sprite = flag;
			break;
		default:
			return;
	}
	Sprite8(x,y,8,clear,LCD_MEM,SPRT_AND);
	Sprite8(x,y,8,sprite,LCD_MEM,SPRT_OR);
}

void drawNumber(int xCoord, int yCoord, int num)
{
	int x = 4;
	x += xCoord * 9;
	int y = 2;
	y += yCoord * 9;
	Sprite8(x,y,8,clear,LCD_MEM,SPRT_AND);
	short oldFont = FontSetSys(F_6x8);
	char buffer[3];
	sprintf(buffer,"%d",num);
	DrawChar(x,y,buffer[0],A_REPLACE);
	FontSetSys(oldFont);
}

void uncoverAll(int** board)
{
	int i, j;
	for (i = 0; i < 9; ++i)
	{
		for (j = 0; j < 17; ++j)
		{
			if (board[j][i] == MINE)
				drawSprite(j,i,MINE);
			else if (board[j][i] <= 8) // is a number or clear
			{
				if (board[j][i] > 0)
					drawNumber(j,i,board[j][i]);
				else
					drawSprite(j,i,CLEAR);
			}
		}
	}
}
