// game.c
//
// 09-06-2003 23:19:32
//
// Copyright 2003 Terry Roberts
//
// See readme.html for more information.
//
// This file is part of MineSweeper.
// 
// MineSweeper is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// MineSweeper is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with MineSweeper; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

// C Source File
// Created 9/6/03; 9:43:57 PM

#include "game.h"
#include "display.h"

void generateBoard(int** board, int numMines)
{
	randomize();
	int i,j,x,y;
	for (i = 0; i < numMines; ++i) // seed the board with mines
	{
		x = random(17);
		y = random(9);
		if (board[x][y] == MINE) // check for duplicates
			--i;
		else
			board[x][y] = MINE;
	}
	for (i = 0; i < 9; ++i) // build numbers for each unmined cell
	{
		for (j = 0; j < 17; ++j)
		{
			if (board[j][i] != MINE)
				board[j][i] = countMines(j,i,board);
		}
	}
}

int countMines(int xCoord, int yCoord, int** board)
{
	int ret = 0;
	if (yCoord != 0) // check top middle
		if (board[xCoord][yCoord - 1] == MINE)
			++ret;
	if (yCoord != 8) // check bottom middle
		if (board[xCoord][yCoord + 1] == MINE)
			++ret;
	if (xCoord != 0) // check left middle
		if (board[xCoord - 1][yCoord] == MINE)
			++ret;
	if (xCoord != 16) // check right middle
		if (board[xCoord + 1][yCoord] == MINE)
			++ret;
	if (xCoord != 0 && yCoord != 0) // check top left
		if (board[xCoord -1][yCoord -1] == MINE)
			++ret;
	if (xCoord != 16 && yCoord != 0) // check top right
		if (board[xCoord + 1][yCoord - 1] == MINE)
			++ret;
	if (xCoord != 0 && yCoord != 8) // check bottom left
		if (board[xCoord - 1][yCoord + 1] == MINE)
			++ret;
	if (xCoord != 16 && yCoord != 8) // check bottom right
		if (board[xCoord + 1][yCoord + 1] == MINE)
			++ ret;
	return ret;
}

int game(int** board, int numMines)
{
	int player = 1,
		p1Flagged = 0, p2Flagged = 0,
		minesLeft = numMines,
		minesToWin = ((numMines + 1) / 2),
		x = 0, y = 0,
		i,j;
	int** played = (int**)malloc(17*sizeof(int**));
	for (i = 0; i < 17; ++i)
		played[i] = (int*)malloc(9*sizeof(int*));
	for (i = 0; i < 9; ++i)
	{
		for (j = 0; j < 17; ++j)
		{
			played[j][i] = 0;
		}
	}
	clrscr();
	drawBoard();
	drawPlayer(player);
	drawMinesLeft(minesLeft);
	drawMinesFlagged(1,p1Flagged);
	drawMinesFlagged(2,p2Flagged);
	
	while (1) // the game loop
	{
		if (p1Flagged >= minesToWin || p2Flagged >= minesToWin)
			break;
		drawSprite(x,y,SELECTED);
		short key = ngetchx();
		if (key == KEY_ESC)
		{
			for (i = 0; i < 17; ++i)
				free(played[i]);
			uncoverAll(board);
			ngetchx();
			return 0;
		}
		else if (key == KEY_ENTER)
		{
			if (played[x][y] == 0)
			{
				if (openCell(x,y,board,played) == MINE)
				{
					if (player == 1)
					{
						++p1Flagged;
						drawMinesFlagged(1,p1Flagged);
					}
					else
					{
						++p2Flagged;
						drawMinesFlagged(2,p2Flagged);
					}
					--minesLeft;
					drawMinesLeft(minesLeft);
				}
				else
				{
					if (player == 1)
						player = 2;
					else
						player = 1;
					drawPlayer(player);
				}
			}
		}
		else if (key == KEY_UP)
		{
			if (played[x][y] == 0)
				drawSprite(x,y,VIRGIN);
			else if (board[x][y] == 0)
				drawSprite(x,y,CLEAR);
			else if (board[x][y] == MINE)
				drawSprite(x,y,FLAG);
			else
				drawNumber(x,y,board[x][y]);
			if (y == 0)
				y = 8;
			else
				--y;
		}
		else if (key == KEY_DOWN)
		{
			if (played[x][y] == 0)
				drawSprite(x,y,VIRGIN);
			else if (board[x][y] == 0)
				drawSprite(x,y,CLEAR);
			else if (board[x][y] == MINE)
				drawSprite(x,y,FLAG);
			else
				drawNumber(x,y,board[x][y]);
			if (y == 8)
				y = 0;
			else
				++y;
		}
		else if (key == KEY_LEFT)
		{
			if (played[x][y] == 0)
				drawSprite(x,y,VIRGIN);
			else if (board[x][y] == 0)
				drawSprite(x,y,CLEAR);
			else if (board[x][y] == MINE)
				drawSprite(x,y,FLAG);
			else
				drawNumber(x,y,board[x][y]);
			if (x == 0)
				x = 16;
			else
				--x;
		}
		else if (key == KEY_RIGHT)
		{
			if (played[x][y] == 0)
				drawSprite(x,y,VIRGIN);
			else if (board[x][y] == 0)
				drawSprite(x,y,CLEAR);
			else if (board[x][y] == MINE)
				drawSprite(x,y,FLAG);
			else
				drawNumber(x,y,board[x][y]);
			if (x == 16)
				x = 0;
			else
				++x;
		}
	} // end while (1)
	for (i = 0; i < 17; ++i)
		free(played[i]);
	uncoverAll(board); // after game, reveal all mines
	ngetchx();
	return player; // return winning player
}

int openCell(int xCoord, int yCoord, int** board, int** played)
{
	int cell = board[xCoord][yCoord];
	if (played[xCoord][yCoord] == 0)
		played[xCoord][yCoord] = 1;
	else
		return cell;
	switch (cell)
	{
		case MINE:
			drawSprite(xCoord,yCoord,FLAG);
			break;
		case 0: // is empty cell, not near any mines
			drawSprite(xCoord,yCoord,CLEAR);
			if (yCoord != 0) // open top middle
				openCell(xCoord, yCoord - 1, board, played);
			if (yCoord != 8) // open bottom middle
				openCell(xCoord, yCoord + 1, board, played);
			if (xCoord != 0) // open left middle
				openCell(xCoord - 1, yCoord, board, played);
			if (xCoord != 16) // open right middle
				openCell(xCoord + 1, yCoord, board, played);
			if (xCoord != 0 && yCoord != 0) // open top left
				openCell(xCoord - 1, yCoord - 1, board, played);
			if (xCoord != 16 && yCoord != 0) // open top right
				openCell(xCoord + 1, yCoord - 1, board, played);
			if (xCoord != 0 && yCoord != 8) // open bottom left
				openCell(xCoord - 1, yCoord + 1, board, played);
			if (xCoord != 16 && yCoord != 8) // open bottom right
				openCell(xCoord + 1, yCoord + 1, board, played);
			break;
		default:
			drawNumber(xCoord,yCoord,cell);
			break;
	}
	return cell;
}
